"""Podcast searching."""

from dataclasses import dataclass
from typing import List, Optional
import json
import urllib.request
import urllib.parse
from urllib.error import HTTPError, URLError

SEARCH_URL = "https://itunes.apple.com/search"
URL_TEMPLATE = "https://podcasts.apple.com/us/podcast/id{}"


@dataclass
class Podcast:
    """Podcast metadata."""

    # see https://github.com/schemaorg/schemaorg/issues/373
    id: str
    name: str
    author: str
    url: str
    feed: Optional[str] = None
    category: Optional[str] = None
    image: Optional[str] = None


def search(name: str, limit: int = 5) -> List[Podcast]:
    """Search podcast by name."""
    params = {"term": name, "limit": limit, "media": "podcast"}
    response = _get(url=SEARCH_URL, params=params)
    return _parse(response)


def _get(url: str, params: Optional[dict] = None) -> dict:
    try:
        qs = urllib.parse.urlencode(params or {})
        req = urllib.request.Request(f"{url}?{qs}")
        with urllib.request.urlopen(req) as response:
            return json.loads(response.read())
    except HTTPError as exc:
        raise Exception(f"HTTP error {exc.code}: {exc.reason}")
    except URLError as exc:
        raise Exception(f"Network error: {exc.reason}")
    except json.JSONDecodeError as exc:
        raise Exception(f"Failed to parse response: {exc}")


def _parse(response: dict) -> List[Podcast]:
    result_count = response.get("resultCount")
    if not result_count:
        return []
    return [_parse_item(item) for item in response.get("results", [])]


def _parse_item(item: dict) -> Podcast:
    try:
        id_ = str(item["collectionId"])
        name = item["collectionName"]
        author = item["artistName"]
        url = URL_TEMPLATE.format(id_)
        podcast = Podcast(id=id_, name=name, author=author, url=url)
        podcast.feed = item.get("feedUrl")
        podcast.category = item.get("primaryGenreName")
        podcast.image = item.get("artworkUrl600")
        return podcast
    except LookupError as exc:
        raise Exception(f"Failed to parse podcast item: {exc}\n{item}")



#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['podsearch']

package_data = \
{'': ['*']}

setup(name='podsearch',
      version='0.1.1',
      description="Let's find some podcasts!",
      author='Anton Zhiyanov',
      author_email='m@antonz.ru',
      url='https://github.com/nalgeon/podsearch-py',
      packages=packages,
      package_data=package_data,
      python_requires='>=3.6',
     )



from unittest.mock import patch
from urllib.error import HTTPError
import pytest
import podsearch


def test_search():
    with patch("podsearch.searcher._get") as mock:
        mock.return_value = {
            "resultCount": 2,
            "results": [
                {
                    "collectionId": 979020229,
                    "artistName": "Michael Kennedy (@mkennedy)",
                    "collectionName": "Talk Python To Me",
                    "collectionViewUrl": "https://podcasts.apple.com/us/podcast/id979020229?uo=4",
                    "feedUrl": "https://talkpython.fm/episodes/rss",
                    "primaryGenreName": "Technology",
                    "artworkUrl600": "https://whatever/image/979020229.png",
                },
                {
                    "collectionId": 981834425,
                    "artistName": "Tobias Macey",
                    "collectionName": "The Python Podcast.__init__",
                    "collectionViewUrl": "https://podcasts.apple.com/us/podcast/id981834425?uo=4",
                    "feedUrl": "https://www.podcastinit.com/feed/mp3/",
                    "primaryGenreName": "Technology",
                    "artworkUrl600": "https://whatever/image/981834425.png",
                },
            ],
        }

        podcasts = podsearch.search("Python")
        assert len(podcasts) == 2
        talkpython = podcasts[0]
        assert talkpython.id == "979020229"
        assert talkpython.name == "Talk Python To Me"
        assert talkpython.author == "Michael Kennedy (@mkennedy)"
        assert talkpython.url == "https://podcasts.apple.com/us/podcast/id979020229"
        assert talkpython.feed == "https://talkpython.fm/episodes/rss"
        assert talkpython.category == "Technology"
        assert talkpython.image == "https://whatever/image/979020229.png"


def test_nothing_found():
    with patch("podsearch.searcher._get") as mock:
        mock.return_value = {"resultCount": 1, "results": []}
        podcasts = podsearch.search("Python")
        assert len(podcasts) == 0


def test_failed():
    with patch("podsearch.searcher._get") as mock:
        mock.side_effect = HTTPError(
            "https://itunes.apple.com/search", 503, "Service Unavailable", {}, None
        )
        with pytest.raises(Exception):
            podsearch.search("Python")
